# -*- coding: utf-8 -*-

class CircularDependencyError(Exception):
    pass

class Node:
    def __init__(self, name, depends):
        self.name = name
        self.depends = set(depends)

# "Batches" are sets of tasks that can be run together
def solve_batches(nodes):
    # Build a map of node names to node instances
    name_to_instance = {n.name: n for n in nodes}
    # Build a map of node names to dependency names
    name_to_deps = {n.name: n.depends.copy() for n in nodes}
    # This is where we'll store the batches
    batches = []
    # While there are dependencies to solve...
    while name_to_deps:
        # Get all nodes with no dependencies
        ready = {name for name, deps in name_to_deps.items() if not deps}
        # If there aren't any, we have a loop in the graph
        if not ready:
            raise CircularDependencyError(name_to_deps)
        # Remove them from the dependency graph
        for name in ready:
            del name_to_deps[name]
        for deps in name_to_deps.values():
            deps.difference_update(ready)
        # Add the batch to the list
        batches.append(ready)
    return batches

def solve_flat(nodes):
    batches = solve_batches(nodes)
    return [i for b in batches for i in b]



a = Node("a", [])
b = Node("b", [])
c = Node("c", ["a"])
d = Node("d", ["b"])
e = Node("e", ["c", "d"])
f = Node("f", ["a", "b"])
g = Node("g", ["e", "f"])
h = Node("h", ["g"])
i = Node("i", ["a"])
j = Node("j", ["b"])
k = Node("k", [])
nodes = (a, b, c, d, e, f, g, h, i, j, k)

# Show the batches on screen
print "Batches:"
for bundle in get_task_batches(nodes):
    print ", ".join(node.name for node in bundle)
print

# An example, *broken* dependency graph
a = Task("a", "i")
nodes = (a, b, c, d, e, f, g, h, i, j)

# Show it on screen
print "A broken dependency graph example:"
print format_nodes(nodes)
print

# This should raise an exception and show the current state of the graph
print "Trying to resolve the dependencies will raise an exception:"
print
get_task_batches(nodes)